/*
  Copyright (C) MOXA Inc. All rights reserved.
  This software is distributed under the terms of the
  MOXA License.  See the file COPYING-MOXA for details.
*/

/*
 * History:
 * Date		Author			Comment
 * 12-06-2005	Victor Yu.		Create it.
 */
#include	<stdio.h>
#include	<stdlib.h>
#include	<fcntl.h>
#include	"ivtcdevice.h"

// following about GPIO API implement
#define GPIO_DEVICE_NODE	"/dev/gpio"

#define IOCTL_GPIO_GET_MODE	1
#define IOCTL_GPIO_SET_MODE	2
#define IOCTL_GPIO_GET_DATA	3
#define IOCTL_GPIO_SET_DATA	4

#define CHECK_GPIO_NO(p) { \
	if ( (p) >= MAX_GPIO ) \
		return GPIO_NO_ERROR; \
}
#define CHECK_GPIO_MODE(m) { \
	if ( (m) != GPIO_INPUT && (m) != GPIO_OUTPUT ) \
		return GPIO_MODE_ERROR; \
}
#define CHECK_GPIO_DATA(d) { \
	if ( (d) != GPIO_HIGH && (d) != GPIO_LOW ) \
		return GPIO_DATA_ERROR; \
}

typedef struct	gpio_set_struct {
	int	io_number;
	int	mode_data;
} gpio_t;

/*
 * To get the GPIO mode now.
 * Input: unsigned int pio	- the GPIO number, from 0 to MAX_GPIO-1
 * Output: < 0			- some error
 * 	   1			- input
 * 	   0			- ouput
 */
int get_gpio_mode(unsigned int gpio_no)
{
	int	fd;
	gpio_t	pset;

	CHECK_GPIO_NO(gpio_no);
	fd = open(GPIO_DEVICE_NODE, O_RDWR);
	if ( fd < 0 )
		return GPIO_NODE_ERROR;
	pset.io_number = gpio_no;
	if ( ioctl(fd, IOCTL_GPIO_GET_MODE, &pset) != 0 ) {
		close(fd);
		return GPIO_ERROR;
	}
	close(fd);
	return pset.mode_data;
}

/*
 * To get the GPIO data now.
 * Input: unsigned int pio	- the GPIO number, from 0 to MAX_GPIO-1
 * Output: < 0			- some error
 * 	   1			- high
 * 	   0			- low
 */
int get_gpio_data(unsigned int gpio_no)
{
	int	fd;
	gpio_t	pset;

	CHECK_GPIO_NO(gpio_no);
	fd = open(GPIO_DEVICE_NODE, O_RDWR);
	if ( fd < 0 )
		return GPIO_NODE_ERROR;
	pset.io_number = gpio_no;
	if ( ioctl(fd, IOCTL_GPIO_GET_DATA, &pset) != 0 ) {
		close(fd);
		return GPIO_ERROR;
	}
	close(fd);
	return pset.mode_data;
}

/*
 * To set the GPIO now mode.
 * Input: unsigned int pio	- the GPIO number, from 0 to MAX_GPIO-1
 * 	  int mode		- want to set mode, 1 for input, 0 for output
 * Output: < 0			- some error
 * 	   = 0			- OK
 */
int set_gpio_mode(unsigned int gpio_no, int mode)
{
	int	fd;
	gpio_t	pset;

	CHECK_GPIO_NO(gpio_no);
	CHECK_GPIO_MODE(mode);
	fd = open(GPIO_DEVICE_NODE, O_RDWR);
	if ( fd < 0 )
		return GPIO_NODE_ERROR;
	pset.io_number = gpio_no;
	pset.mode_data = mode;
	if ( ioctl(fd, IOCTL_GPIO_SET_MODE, &pset) != 0 ) {
		close(fd);
		return GPIO_ERROR;
	}
	close(fd);
	return GPIO_OK;
}

/*
 * To set the GPIO now data.
 * Input: unsigned int pio	- the GPIO number, from 0 to MAX_GPIO-1
 * 	  int data		- 1 for high, 0 for low
 * Output: < 0			- some error
 * 	   = 0			- OK
 */
int set_gpio_data(unsigned int gpio_no, int data)
{
	int	fd;
	gpio_t	pset;

	CHECK_GPIO_NO(gpio_no);
	CHECK_GPIO_DATA(data);
	fd = open(GPIO_DEVICE_NODE, O_RDWR);
	if ( fd < 0 )
		return GPIO_NODE_ERROR;
	pset.io_number = gpio_no;
	pset.mode_data = data;
	if ( ioctl(fd, IOCTL_GPIO_SET_DATA, &pset) != 0 ) {
		close(fd);
		return GPIO_ERROR;
	}
	close(fd);
	return GPIO_OK;
}
